
{*****************************************************************************
 * Name           : KNGDBControls                                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 06/07/1999                                               *
 * Description    :                                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 06/07/1999 Lesage Stefaan  Initial creation of the Unit.                  *
 *****************************************************************************}


unit KNGDBControls;

{$I KNG.INC}

interface

uses Buttons, Classes, Controls, DB, DBCtrls, ExtCtrls, Messages,
    {$IFDEF KNG_D4} ImgList, {$ENDIF} Windows, KNGControls;


const
  InitRepeatPause = 400;
  RepeatPause     = 100;

type
  TKNGNavButton = class;
  TKNGNavDataLink = class;

  TKNGNavGlyph = (ngEnabled, ndDisabled);

  TKNGNavBtns = (nbFirst, nbPriorSet, nbPrior, nbNext, nbNextSet, nbLast,
                 nbInsert, nbDelete, nbEdit, nbPost, nbCancel, nbRefresh,
                 nbSetBookmark, nbGotoBookmark, nbClearBookmark, nbEnterFilter, nbApplyFilter, nbCancelFilter,
                 nbCopyRecord,
                 nbPrintList, nbPrintRecord);
  TKNGNavBtnSet = set of TKNGNavBtns;

  TKNGNavBtnStyle = set of (nsAllowTimer, nsFocusRect);
  TKNGNavOrientation = (orHorizontal, orVertical);

  EKNGNavClick = procedure (Sender: TObject; Button: TKNGNavBtns) of object;
  EKNGNavBefore = procedure(Sender: TObject; var Button: TKNGNavBtns;
                                             var ProcessAction: Boolean) of object;

  TKNGDBNavigator = class(TKNGCustomPanel)
  private
    { Private Variables }
    ButtonWidth: Integer;
    ButtonHeight: Integer;
    FocusedButton: TKNGNavBtns;
    MinBtnSize: TPoint;

    { Private Property Variables }
    FAlign: TAlign;
    FBookmark: TBookmark;
    FDefaultHints: TStrings;
    FConfirmCancel: Boolean;
    FConfirmCancelText: string;
    FConfirmDelete: Boolean;
    FConfirmDeleteText: string;
    FDataLink: TKNGNavDataLink;
    FFlat: Boolean;
    FFocusEditControl: TWinControl;
    FGroupSpace: Integer;
    FHints: TStrings;
    FMoveBySet : Integer;
    FOrientation: TKNGNavOrientation;

    FVisibleButtons : TKNGNavBtnSet;
    FEnabledButtons : TKNGNavBtnSet;

    { Events }
    FBeforeAction: EKNGNavBefore;
    FOnClick: EKNGNavClick;

    {$IFDEF WIN32}
    FImages: TImageList;
    FImageChangeLink: TChangeLink;
    procedure ImageListChange(Sender: TObject);
    procedure SetImages(Value: TImageList);
    {$ENDIF}

    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure AdjustSize(var W: Integer; var H: Integer);
    procedure BtnMouseDown(Sender: TObject; Button: TMouseButton;
                           Shift: TShiftState; X, Y: Integer);
    procedure ClickHandler(Sender: TObject);
    procedure CopyRecord;
    procedure ShowImages;
    procedure HintsChanged(Sender: TObject);
    procedure InitButtons;
    procedure InitHints;
    procedure UpdateEnabled;
    procedure SetAlign(Value: TAlign);
    procedure SetFlat(Value: Boolean);
    procedure SetFocusEditControl(Value: TWinControl);
    procedure SetGroupSpace(Value: Integer);
    procedure SetHints(Value: TStrings);
    function GetHints: TStrings;
    procedure SetMoveBySet(Value: Integer);
    procedure SetOrientation(Value: TKNGNavOrientation);
    procedure SetVisibleButtons(Value: TKNGNavBtnSet);
    procedure SetFocusToEditControl;

    { Windows Messages for Resizing }
    procedure WMSize(var Message: TWMSize); message WM_Size;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    { Protected declarations }
    Buttons : array[TKNGNavBtns] of TKNGNavButton;
    procedure ActiveChanged;
    procedure BevelChanged(Sender: TObject); override;
    procedure DataChanged;
    procedure EditingChanged;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    { Constructor and Destructor }
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BtnClick(Index: TKNGNavBtns);
    procedure FreeBookmark; virtual;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    property EnabledBottons: TKNGNavBtnSet read FEnabledButtons;
  published
    { Published declarations }
    { Properties }
    property About;
    property Align read FAlign write SetAlign default alNone;
    property Bevel;
    property ConfirmCancel: Boolean read FConfirmCancel write FConfirmCancel default False;
    property ConfirmCancelText: string read FConfirmCancelText write FConfirmCancelText;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete default True;
    property ConfirmDeleteText: string read FConfirmDeleteText write FConfirmDeleteText;
    property Ctl3D;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Flat: Boolean read FFlat write SetFlat default False;
    property FocusEditControl: TWinControl read FFocusEditControl write SetFocusEditControl;
    property Images: TImageList read FImages write SetImages;
    property GroupSpace: Integer read FGroupSpace write SetGroupSpace default 10;
    property Hints: TStrings read GetHints write SetHints;
    property MoveBySet: Integer read FMoveBySet write SetMoveBySet;
    property Orientation: TKNGNavOrientation read FOrientation write SetOrientation default orHorizontal;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property VisibleButtons: TKNGNavBtnSet read FVisibleButtons write SetVisibleButtons;
    { Events }
    property BeforeAction: EKNGNavBefore read FBeforeAction write FBeforeAction;
    property OnClick: EKNGNavClick read FOnClick write FOnClick;
    property OnMouseEnter;
    property OnMouseLeave;
  end;

  TKNGNavButton = Class(TSpeedButton)
  private
    FIndex: TKNGNavBtns;
    FNavStyle: TKNGNavBtnStyle;
    FRepeatTimer: TTimer;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    destructor Destroy; override;
    property NavStyle: TKNGNavBtnStyle read FNavStyle write FNavStyle;
    property Index: TKNGNavBtns read FIndex write FIndex;
  end;

  TKNGNavDataLink = Class(TDataLink)
  private
    FNavigator: TKNGDBNavigator;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(ANav: TKNGDBNavigator);
    destructor Destroy; override;
  end;

  TKNGDBText = class(TKNGCustomLabel)
  private
    FDataLink: TFieldDataLink;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetFieldText: string;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    function GetLabelText: string; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetAutoSize(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
  published
    property About;
    property Align;
    property Alignment;
    property AutoSize default False;
    property Bevel;
    property Color;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Layout;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TKNGStatusDataLink = class;
  TKNGDBStatusLabel = class;

  TKNGDBStatusLabel = class(TKNGCustomLabel)
  private
    FDataLink: TDataLink;
    FDataBaseName: string;
    FShowDatabaseName: Boolean;
    function GetDatabaseName: string;
    procedure SetDatabaseName(Value: string);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetShowDatabaseName(Value: Boolean);
  protected
    procedure Loaded; override;
    procedure SetName(const Value: TComponentName); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateCaption; virtual;
    property Caption;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DatabaseName: string read GetDatabaseName write SetDatabaseName;
    property ShowDatabaseName: Boolean read FShowDatabaseName write SetShowDatabaseName;

    { Properties from TCustomLabel }

    property About;
    property Align;
    property Alignment;
    property AutoSize;
    property Bevel;
    property Color;
    property DragCursor;
    property DragMode;
    property Font;
    property Layout;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Tag;
    property Transparent;
    property Visible;
    property WordWrap;

    { Events from TCustomLabel }
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseUp;
    property OnStartDrag; 
  end;

  TKNGStatusDataLink = class(TDataLink)
  private
    FKNGDBStatusLabel : TKNGDBStatusLabel;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
    procedure EditingChanged; override;
    procedure LayoutChanged; override;
  public
    constructor Create(AKNGDBStatusLabel: TKNGDBStatusLabel);
    destructor Destroy; override;
  end;

  TKNGDBLookupLabel = Class;

  TSourceDataLink = class(TDataLink)
  private
    FKNGDBLookupLabel: TKNGDBLookupLabel;
  protected
    procedure FocusControl(Field: TFieldRef); override;
    procedure ActiveChanged; override;
    procedure RecordChanged(Field: TField); override;
  end;

  TLookupDataLink = class(TDataLink)
  private
    FKNGDBLookupLabel: TKNGDBLookupLabel;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
  end;

  { TKNGDBLookupLabel }

  TKNGDBLookupLabel = class(TKNGCustomLabel)
  private
    FDataField: TField;
    FDataFieldName: string;
    FDataLink: TSourceDataLink;
    FDisplayEmpty: string;
    FDisplayIndex : integer;
    FDisplayField : TField;
    FDisplayValue : string;
    FEmptyValue: string;
    FKeyField: TField;
    FListActive: Boolean;
    FListFields: TList;
    FLookupDisplay: string;
    FLookupFieldName : string;
    FLookupLink: TLookupDataLink;
    FLookupMode: Boolean;
    FLookupSource : TDataSource;
    FMasterField: TField;
    FValue: string;
    procedure CheckNotFixed;
    procedure CheckNotCircular;
    procedure DataLinkActiveChanged;
    procedure DataLinkRecordChanged(Field: TField);
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetKeyValue: Variant;
    function GetLabelText: string; override;
    function GetLookupField: string;
    function GetLookupSource: TDataSource;
    procedure KeyValueChanged;
    procedure ListLinkActiveChanged;
    procedure ListLinkDataChanged;
    procedure SetDataFieldName(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetDisplayEmpty(const Value : string);
    procedure SetEmptyValue(const Value: string);
    procedure SetLookupDisplay(const Value: string);
    procedure SetLookupField(const Value: string);
    procedure SetLookupMode(Value: Boolean);
    procedure SetLookupSource(Value: TDataSource);
    procedure SetValueKey(const Value: string);
    function ValueIsEmpty(const s: string): Boolean;
  protected
    property KeyValue: Variant read GetKeyValue stored False;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure UpdateDisplayEmpty(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
    property Value: string read FValue;
  published
    property About;
    property Align;
    property Alignment;
    property AutoSize default False;
    property Bevel;
    property Color;
    property DataField: string read FDataFieldName write SetDataFieldName;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DisplayEmpty: string read FDisplayEmpty write SetDisplayEmpty;
    property DragCursor;
    property DragMode;
    property EmptyValue: string read FEmptyValue write SetEmptyValue; //stored StoreEmpty;
    property Enabled;
    property Font;
    property Layout;
    property LookupDisplay: string read FLookupDisplay write SetLookupDisplay;
    property LookupDisplayIndex: integer read FDisplayIndex write FDisplayIndex default 0;
    property LookupField: string read GetLookupField write SetLookupField;
    property LookupSource: TDataSource read GetLookupSource write SetLookupSource;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Transparent;
    property ShowHint;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseUp;
    property OnStartDrag;
  end;


implementation

uses SysUtils, Dialogs, KNGConst, DBTables;

{$R KNGDB.RES}

var
  BtnTypeName: array [TKNGNavBtns] of PChar =
    ('FIRST', 'PRIORSET', 'PRIOR', 'NEXT', 'NEXTSET', 'LAST',
     'INSERT', 'DELETE', 'EDIT', 'POST', 'CANCEL', 'REFRESH',
     'SETBOOKMARK', 'GOTOBOOKMARK', 'CLEARBOOKMARK', 'ENTERFILTER', 'APPLYFILTER', 'CANCELFILTER',
     'COPYRECORD',
     'PRINTLIST', 'PRINTRECORD');
  BtnHintID: array [TKNGNavBtns] of Pointer =
    (@sFirstRecord, @sPriorSet, @sPriorRecord, @sNextRecord, @sNextSet, @sLastRecord,
     @sInsertRecord, @sDeleteRecord, @sEditRecord, @sPostEdit, @sCancelEdit, @sRefreshRecord,
     @sSetBookMark, @sGotoBookMark, @sClearBookMark, @sEnterFilter, @sApplyFilter, @sCancelFilter,
     @sCopyRecord,
     @sPrintList, @sPrintRecord);

{*****************************************************************************
 * Name           : TKNGDBNavigator.Create                                   *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 18/06/1999                                               *
 * Arguments      : AOwner.  The owner of the component                      *
 * Return Values  : None                                                     *
 * Description    : This is the constructor of the DBNavigator.  In here we  *
 *                  will create other components and set some default        *
 *                  properties                                               *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 06/07/1999 Lesage Stefaan  Component now deschends from TKNGCustomPanel   *
 *                            so it owns new properties.                     *
 * 18/06/1999 Lesage Stefaan  Modified the Constructor to incorporate the    *
 *                            ButtonGlyphs property.                         *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

constructor TKNGDBNavigator.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);

  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [csOpaque];
  if not NewStyleControls then ControlStyle := ControlStyle + [csFramed];

  { Create The DataLink }
  FDataLink := TKNGNavDataLink.Create(Self);

  { Set the Default Visible Buttons }
  FVisibleButtons := [nbFirst, nbPrior, nbNext, nbLast,
                      nbInsert, nbDelete, nbEdit, nbPost, nbCancel, nbRefresh];
  FEnabledButtons := FVisibleButtons;

  { String List for Hints }
  FHints:= TStringList.Create;
  TStringList(FHints).OnChange := HintsChanged;

  { Set the default space between groups of buttons }
  FGroupSpace := 10;

  { Create the ImageList Change Link.  This is necessary because we want to
    get notified if the ImageList changes, and we want to redraw the buttons
    at that moment }
  {$IFDEF WIN32}
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  {$ENDIF}

  { Set some properties from the Bevel Object }
  Bevel.BevelInner := bvNone;
  Bevel.BevelOuter := bvNone;

  { Initialise the Buttons }
  InitButtons;

  { Set some default properties }
  Width :=150;
  Height := 30;
  FMoveBySet := 10;
  FConfirmDelete := True;
  FConfirmCancel := False;
  FOrientation := orHorizontal;
  FBookMark := nil;

  { Fill the ConfirmDeleteText and ConfirmCancelText with the Value
    of sConfirmDeleteQuestion and sConfirmCancelQuestion for the
    language specified in the file KNG.INC }
  FConfirmCancelText := sConfirmCancelQuestion;
  FConfirmDeleteText := sConfirmDeleteQuestion;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.Destroy                                  *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 18/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the Destructor of the component.  In here we     *
 *                  should free all objects we created and clean up.         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 18/06/1999 Lesage Stefaan  Modified the Destructor so that it also frees  *
 *                            the ButtonGlyphsChangeLink.                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

destructor TKNGDBNavigator.Destroy;
begin
  FreeBookmark;

  FDataLink.Free;
  FDataLink := nil;

  FDefaultHints.Free;
  FDefaultHints := nil;

  FHints.Free;
  FHints := nil;

  FImages := nil;

  FImageChangeLink.OnChange := nil;
  FImageChangeLink.Free;
  FImageChangeLink := nil;

  inherited Destroy;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.ImageListChange                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 18/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is executed when the ImageList changes.   *
 *                  In here we will call the ShowButtonGlyps procedure to    *
 *                  update the Glyphs on the TKNGDBNavigator buttons.        *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 18/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.ImageListChange(Sender: TObject);
var
  tmpButtonGlyphs: TImageList;
begin
  tmpButtonGlyphs := FImages;
  FImages := nil;
  ShowImages;
  FImages := tmpButtonGlyphs;
  ShowImages;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetImages                                *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 18/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is called when the user sets the Button-  *
 *                  glyphs property.                                         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 18/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetImages(Value: TImageList);
begin
  if Assigned(FImages) then
    FImages.UnRegisterChanges(FImageChangeLink);
  FImages := Value;
  if Assigned(FImages) then
    FImages.RegisterChanges(FImageChangeLink);
  ShowImages;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.ShowImages                               *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 18/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the procedure that will actually update the      *
 *                  glyphs of our TKNGDBNavigator buttons.  If no ImageList  *
 *                  is specified then it will use the default Glyphs from the*
 *                  Resource File.  If there is an ImageList assigned to the *
 *                  ButtonGlyphs property it will use the images in the Image*
 *                  list.  The first image will be for the nbFirst button,   *
 *                  the second for the nbPriorSet button and so on.  If there*
 *                  arn't enough images in the ImageList the glyphs from the *
 *                  resource file will be used for the remaining buttons.    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 18/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.ShowImages;
var
  I: TKNGNavBtns;
  j: Integer;
  ResName: string;
begin
  J := 0;
  for i := Low(Buttons) to High(Buttons) do
  begin
    FmtStr(ResName, 'KNGNB_%s', [BtnTypeName[I]]);
    if not Assigned(FImages) then
    begin
      Buttons[I].Glyph.LoadFromResourceName(HInstance, Resname);
      Buttons[I].NumGlyphs := 2
    end
    else
    begin
      if (J < FImages.Count) then
      begin
        Buttons[I].Glyph := nil;
        FImages.GetBitmap(J, Buttons[I].Glyph);
        Buttons[I].NumGlyphs := 2;
      end;
    end;
    Inc(j);
    Invalidate;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.ActiveChanged                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 04/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the associated dataset's *
 *                  active property gets changed.  So here we should disable *
 *                  all buttons if the Active property is False              *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.ActiveChanged;
var
  I : TKNGNavBtns;
begin
  if not (Enabled and FDataLink.Active) then
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Enabled := False
  else
  begin
    DataChanged;
    EditingChanged;
  end;
  UpdateEnabled;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.AdjustSize                               *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 14/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when buttons are made visible *
 *                  or invisible, when the size of the DBNavigator gets      *
 *                  changed, so here we will resize the buttons if necessary *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 14/06/1999 Lesage Stefaan  Added grouping for Print Buttons               *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.AdjustSize (var W: Integer; var H: Integer);
var
  SpcCount: Integer;
  Count: Integer;
  MinW, MinH: Integer;
  I: TKNGNavBtns;
  Space, Temp, Remain: Integer;
  X, Y: Integer;


  OtherGroupSpace : Boolean;
  ActionGroupSpace : Boolean;
  SearchGroupSpace : Boolean;
  PrintGroupSpace : Boolean;
begin
  OtherGroupSpace := False;
  ActionGroupSpace := False;
  SearchGroupSpace := False;
  PrintGroupSpace := False;

  if (csLoading in ComponentState) then Exit;
  if Buttons[nbFirst] = nil then Exit;

  { First We need to count how many buttons are Visible in the Navigator }
  Count := 0;
  SpcCount := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    if Buttons[I].Visible then
    begin
      Inc(Count);
    end;
  end;
  if Count = 0 then Inc(Count);

  { Now check if One of the Action buttons is visible, so that we can Add
    a GroupSpace before the first button.  To do this we will increment the
    counter by one. }
  if (nbInsert in VisibleButtons) or (nbDelete in VisibleButtons) or
     (nbEdit in VisibleButtons) or (nbPost in VisibleButtons) or
     (nbCancel in VisibleButtons) or (nbRefresh in VisibleButtons) then
  begin
    ActionGroupSpace := True;
    Inc(SpcCount);
  end;

  { Now check if One of the Search of Filter buttons is visible, so that we can
    Add a GroupSpace before the first button.  To do this we will increment the
    counter by one. }
  if (nbSetBookmark in VisibleButtons) or (nbGotoBookmark in VisibleButtons) or
     (nbClearBookmark in VisibleButtons) or (nbEnterFilter in VisibleButtons) or
     (nbApplyFilter in VisibleButtons) or (nbCancelFilter in VisibleButtons) then
  begin
    SearchGroupSpace := True;
    Inc(SpcCount);
  end;

  { Now check if One of the Other buttons is visible, so that we can Add
    a GroupSpace before the first button.  To do this we will increment the
    counter by one. }
  if (nbCopyRecord in VisibleButtons) then
  begin
    OtherGroupSpace := True;
    Inc(SpcCount);
  end;

  { Now check if One of the Print buttons is visible, so that we can Add
    a GroupSpace before the first button.  To do this we will increment the
    counter by one. }
  if (nbPrintList in VisibleButtons) or (nbPrintRecord in VisibleButtons) then
  begin
    PrintGroupSpace := True;
    Inc(SpcCount);
  end;

  { Calculate the Minimum Width of our navigator so it can hold all Buttons }

  if FOrientation = orHorizontal then
  begin
    { Calculate the Minimum Size of the DBNavigator using the Number of Buttons
      that need to be displayed and the Size of two Bevel Sides.  If the current
      size of the DBNavigator is smaller than the minimumsize, we should change
      its size }
    MinW := (Count * MinBtnSize.X) + (SpcCount * FGroupSpace) + (2 * (Bevel.BevelSideSize + 1));
    MinH := MinBtnSize.y + (2 * (Bevel.BevelSideSize + 1));

    if W < MinW then W := MinW;
    if H < MinH then H := MinH;

    { Now Calculate the Height and the Width of one Button }
    ButtonHeight := ClientHeight;
    ButtonWidth := (((W - (2 * (Bevel.BevelSideSize + 1))) - (SpcCount * FGroupSpace)) div Count);

    Temp := (Count * ButtonWidth) + (SpcCount * FGroupSpace) + (2 * (Bevel.BevelSideSize + 1));

    if Align = alNone then W := Temp;
    X := Bevel.BevelSideSize + 1;
    Remain := w - Temp;
    Temp := Count div 2;
    for I := Low(Buttons) to High(Buttons) do
    begin
      if Buttons[I].Visible then
      begin
        Space := 0;
        if Remain <> 0 then
        begin
          Dec (Temp, Remain);
          if Temp < 0 then
          begin
            Inc (Temp, Count);
            Space := 1
          end
        end;
        if ((I = nbSetBookmark) or (I = nbGotoBookmark) or
            (I = nbClearBookmark) or (I = nbEnterFilter) or
            (I = nbApplyFilter) or (I = nbCancelFilter)) and
            SearchGroupSpace then
        begin
          Inc(X, GroupSpace);
          SearchGroupSpace := False;
        end;
        if ((I = nbInsert) or (I = nbDelete) or (I = nbEdit) or
            (I = nbPost) or (I = nbCancel) or (I = nbRefresh)) and
            ActionGroupSpace then
        begin
          Inc(X, GroupSpace);
          ActionGroupSpace := False;
        end;
        if ((I = nbCopyRecord)) and OtherGroupSpace then
        begin
          Inc(X, GroupSpace);
          OtherGroupSpace := False;
        end;
        if ((I = nbPrintList) or (I = nbPrintRecord)) and
           PrintGroupSpace then
        begin
          Inc(X, GroupSpace);
          PrintGroupSpace := False;
        end;
        Buttons[I].SetBounds (X, Bevel.BevelSideSize + 1, ButtonWidth + Space, ButtonHeight);
        Inc(X, ButtonWidth + Space);
      end
      else
        Buttons[I].SetBounds (Width + 1, Bevel.BevelSideSize + 1, ButtonWidth, ButtonHeight);
    end;
  end
  else
  begin
    { Calculate the Minimum Size of the DBNavigator using the Number of Buttons
      that need to be displayed and the Size of two Bevel Sides.  If the current
      size of the DBNavigator is smaller than the minimumsize, we should change
      its size }
    MinH := (Count * MinBtnSize.Y) + (SpcCount * FGroupSpace) +
            (2 * (Bevel.BevelSideSize + 1));
    MinW := MinBtnSize.x + (2 * (Bevel.BevelSideSize + 1));

    if H < MinH then H := MinH;
    if W < MinW then W := MinW;

    { Now Calculate the Height and the Width of one Button }
    ButtonHeight := (((H - (2 * (Bevel.BevelSideSize + 1))) - (SpcCount * FGroupSpace)) div Count);
    ButtonWidth := ClientWidth;

    Temp := (Count * ButtonHeight) + (SpcCount * FGroupSpace) +
            (2 * (Bevel.BevelSideSize + 1));

    if Align = alNone then H := Temp;
    Y := Bevel.BevelSideSize + 1 ;
    Remain := H - Temp;
    Temp := Count div 2;
    for I := Low(Buttons) to High(Buttons) do begin
      if Buttons[I].Visible then begin
        Space := 0;
        if Remain <> 0 then begin
          Dec (Temp, Remain);
          if Temp < 0 then begin
            Inc (Temp, Count);
            Space := 1
          end
        end;
        if ((I = nbSetBookmark) or (I = nbGotoBookmark) or
            (I = nbClearBookmark) or (I = nbEnterFilter) or
            (I = nbApplyFilter) or (I = nbCancelFilter)) and
            SearchGroupSpace then
        begin
          Inc(Y, GroupSpace);
          SearchGroupSpace := False;
        end;
        if ((I = nbInsert) or (I = nbDelete) or (I = nbEdit) or
            (I = nbPost) or (I = nbCancel) or (I = nbRefresh)) and
            ActionGroupSpace then
        begin
          Inc(Y, GroupSpace);
          ActionGroupSpace := False;
        end;
        if ((I = nbCopyRecord)) and OtherGroupSpace then
        begin
          Inc(Y, GroupSpace);
          OtherGroupSpace := False;
        end;
        if ((I = nbPrintList) or (I = nbPrintRecord)) and
           PrintGroupSpace then
        begin
          Inc(Y, GroupSpace);
          PrintGroupSpace := False;
        end;
        Buttons[I].SetBounds (Bevel.BevelSideSize + 1, Y, ButtonWidth, ButtonHeight + Space);
        Inc(Y, ButtonHeight + Space);
      end else
        Buttons[I].SetBounds (Bevel.BevelSideSize + 1, Height + 1, ButtonHeight, ButtonWidth);
    end
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.BtnClick                                 *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : Index : TKNGNavBtns which is the button that has been    *
 *                  pressed.
 * Return Values  : None                                                     *
 * Description    : This is the OnClick event for all the buttons on the     *
 *                  DBNavigator, it gets called when a button on the         *
 *                  DBNavigator gets clicked, so here we should respond      *
 *                  with the action associated to the clicked button.        *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  A new property has been added to the compoenent*
 *                            FocusEditControl.  This will hold a pointer to *
 *                            a control which should receive focus when the  *
 *                            user pressed the insert or edit button.        *
 *                            this procedure has been updated so that the    *
 *                            FocusEditControl gets the focus when the user  *
 *                            presses the Edit or Insert button.             *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.BtnClick(Index: TKNGNavBtns);
var
  ProcessAction : Boolean;          { Needed to know if we can still process
                                      the action associated to the pressed
                                      button }
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then
  begin
    { Set ProcessAction default to true }
    ProcessAction := True;
    if not (csDesigning in ComponentState) and Assigned(FBeforeAction) then
      FBeforeAction(Self, Index, ProcessAction);

    { If the ProcessAction set to false in the BeforeAction event, processing
      should stop here, and we should skip to the OnNavAction event }
    if ProcessAction then
    begin
      case Index of
        nbFirst   : DataSource.DataSet.First;
        nbPriorSet: DataSource.DataSet.MoveBy(0-FMoveBySet);
        nbPrior   : DataSource.DataSet.Prior;
        nbNext    : DataSource.DataSet.Next;
        nbNextSet : DataSource.DataSet.MoveBy(FMoveBySet);
        nbLast    : DataSource.DataSet.Last;
        nbInsert  :
        begin
          DataSource.DataSet.Insert;
          SetFocusToEditControl;
        end;
        nbDelete  :
          if (not FConfirmDelete)
          or (MessageDlg(FConfirmDeleteText,
                         mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
            DataSource.DataSet.Delete;
        nbEdit    :
        begin
          DataSource.DataSet.Edit;
          SetFocusToEditControl;
        end;
        nbPost    : DataSource.DataSet.Post;
        nbCancel:
          if (not FConfirmCancel)
          or (MessageDlg(FConfirmCancelText,
                         mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
            DataSource.DataSet.Cancel;
        nbRefresh : DataSource.DataSet.Refresh;
        nbCopyRecord:
        begin
          CopyRecord;
          SetFocusToEditControl;
        end;
        nbSetBookmark:
        begin
          { Free the existing bookmark if there is any }
          if (FBookmark <> nil) then DataSource.DataSet.FreeBookmark(FBookmark);
          FBookMark := DataSource.DataSet.GetBookMark;
          Buttons[nbGotoBookmark].Enabled := True;
          Buttons[nbClearBookmark].Enabled := True;
          Buttons[nbSetBookmark].Enabled := False;
        end;
        nbGotoBookmark:
          if (FBookmark <> nil) then DataSource.DataSet.GotoBookmark(FBookMark);
        nbClearBookmark:
        begin
          if (FBookmark <> nil) then DataSource.DataSet.FreeBookmark(FBookmark);
          FBookmark := nil;
          Buttons[nbSetBookmark].Enabled := True;
          Buttons[nbGotoBookmark].Enabled := False;
          Buttons[nbClearBookmark].Enabled := False;
        end;
        nbEnterFilter:
        begin
          Buttons[nbEnterFilter].Enabled := False;
          Buttons[nbApplyFilter].Enabled := True;
          Buttons[nbCancelFilter].Enabled := DataSource.DataSet.Filtered;
        end;
        nbApplyFilter:
        begin
          Buttons[nbEnterFilter].Enabled := False;
          Buttons[nbApplyFilter].Enabled := False;
          Buttons[nbCancelFilter].Enabled := DataSource.DataSet.Filtered;
        end;
        nbCancelFilter:
        begin
          Buttons[nbEnterFilter].Enabled := True;
          Buttons[nbApplyFilter].Enabled := False;
          Buttons[nbCancelFilter].Enabled := DataSource.DataSet.Filtered;
        end;
      end;
    end;
    UpdateEnabled;
  end;
  if not (csDesigning in ComponentState) and Assigned(FOnClick) then
    FOnClick(Self, Index);
end;

procedure TKNGDBNavigator.BtnMouseDown(Sender: TObject; Button: TMouseButton;
                                       Shift: TShiftState; X, Y: Integer);
var
  OldFocus: TKNGNavBtns;
begin
  OldFocus := FocusedButton;
  FocusedButton := TKNGNavButton(Sender).Index;
  if TabStop and (GetFocus <> Handle) and CanFocus then
  begin
    SetFocus;
    if (GetFocus <> Handle) then
      Exit
  end
  else if TabStop and (GetFocus = Handle) and (OldFocus <> FocusedButton) then
  begin
    Buttons[OldFocus].Invalidate;
    Buttons[FocusedButton].Invalidate;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.CopyRecord                               *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called if the nbCopyRecord button is *
 *                  clicked.  It will copy the values of all fields from the *
 *                  current record into memory, insert a new record and fill *
 *                  the fields with the values that have been copied.  It    *
 *                  won't overwrite default values.                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Jean-Christophe Boggio reported a problem with *
 *                            the CopyRecord funtion.  It seems that auto-   *
 *                            inc fields also get copied, which gives a key  *
 *                            violation error.  Problem has been solved      *
 *                            thanks to Jean-Christophe.                     *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.CopyRecord;
var
  CopyData: Variant;
  Counter: Integer;
begin
  with DataSource.DataSet do
  begin
    { Store the data of the Current record into a Variant Array }
    CopyData := VarArrayCreate([0, FieldCount-1], varVariant);
    for Counter := 0 to FieldCount - 1 do
      CopyData[Counter] := Fields[Counter].Value;

    { Insert a new Record }
    Insert;

    { Insert the copied Data into the new record only if there
      is no default data specified, and if the field isn't a calcluated
      or AutoInc field }
    for Counter := 0 to FieldCount - 1 do
    begin
      if (Fields[Counter].IsNull) and                    { Field is Empty }
         (not Fields[Counter].Calculated) and            { Not a Calc Field }
         (not (Fields[Counter].DataType=ftAutoInc)) then { No AutoInc Field }
        Fields[Counter].Value := CopyData[Counter];
    end;
  end;
end;

procedure TKNGDBNavigator.ClickHandler(Sender: TObject);
begin
  BtnClick(TKNGNavButton(Sender).Index);
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.DataChanged                              *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the data of the DataSet  *
 *                  gets changed.  Anything that changes the contents of the *
 *                  dataset, whether it is editing the data, inserting or    *
 *                  deleting records, filtering, ... triggers this method.   *
 *                  Scrolling the dataset also trigger this method.          *
 *                  So this is a good time to enable or disable some buttons *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  2 Buttons have been added, one for printing a  *
 *                            list of all records, and one for printing the  *
 *                            current recrod                                 *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.DataChanged;
var
  UpEnable, DnEnable, DataEnable, RecordsAvailable : Boolean;
begin
  UpEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.BOF;
  DnEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.EOF;
  DataEnable := Enabled and FDataLink.Active and FDataLink.DataSet.CanModify
                        and not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
  RecordsAvailable := Enabled and FDataLink.Active
                      and not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
  Buttons[nbFirst].Enabled := UpEnable;
  Buttons[nbPriorSet].Enabled := UpEnable;
  Buttons[nbPrior].Enabled := UpEnable;
  Buttons[nbNext].Enabled := DnEnable;
  Buttons[nbNextSet].Enabled := DnEnable;
  Buttons[nbLast].Enabled := DnEnable;
  Buttons[nbDelete].Enabled := DataEnable;
  Buttons[nbSetBookmark].Enabled := (FBookmark = nil);
  Buttons[nbGotoBookmark].Enabled :=(FBookmark <> nil);
  Buttons[nbClearBookmark].Enabled := (FBookmark <> nil);
  Buttons[nbEnterFilter].Enabled := not(DataSource.DataSet.Filtered);
  Buttons[nbApplyFilter].Enabled := not(Buttons[nbEnterFilter].Enabled) and not(DataSource.Dataset.Filtered);
  Buttons[nbCancelFilter].Enabled := DataSource.DataSet.Filtered;
  Buttons[nbPrintList].Enabled := RecordsAvailable;
  Buttons[nbPrintRecord].Enabled := RecordsAvailable;
  UpdateEnabled;
//  if not (csDesigning in ComponentState) and Assigned(FOnDataChanged) then FOnDataChanged(Self);
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.EditingChanged                           *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 04/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This method gets called when the editing state of the    *
 *                  datasource gets changed, so in here we will enable/      *
 *                  disalbe editing buttons depending on the current editing *
 *                  state.                                                   *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.EditingChanged;
var
  CanModify : Boolean;
begin
  CanModify := Enabled and FDataLink.Active and FDataLink.DataSet.CanModify;
  Buttons[nbInsert].Enabled := CanModify;
  Buttons[nbCopyRecord].Enabled := CanModify and not FDataLink.Editing;
  Buttons[nbEdit].Enabled := CanModify and not FDataLink.Editing;
  Buttons[nbPost].Enabled := CanModify and FDataLink.Editing;
  Buttons[nbCancel].Enabled := CanModify and FDataLink.Editing;
  Buttons[nbRefresh].Enabled := CanModify;
  UpdateEnabled;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.FreeBookmark                             *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the procedure that will be used for freeing the  *
 *                  bookmark.                                                *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Bug Correction.  When the bookmark was freed   *
 *                            the GotoBookmark and ClearBookmark buttons were*
 *                            still enabled and the SetBookmark button was   *
 *                            disabled.  After freeing the bookmark it should*
 *                            have been the other way around.                *
 *                            Thanks to Juergen Beyer for reporting this.    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.FreeBookmark;
begin
  if (FBookmark <> nil) then
  begin
    try
      DataSource.DataSet.FreeBookMark(FBookMark);
    finally
      FBookmark := nil;
    end;
  end;
  Buttons[nbSetBookmark].Enabled := (FBookmark <> nil);
  Buttons[nbGotoBookmark].Enabled :=(FBookmark = nil);
  Buttons[nbClearBookmark].Enabled := (FBookmark = nil);
  UpdateEnabled;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.HintsChanged                             *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 04/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the Hints property gets  *
 *                  changed.                                                 *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.HintsChanged(Sender: TObject);
begin
  InitHints;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.InitButtons                              *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 04/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the Component is created *
 *                  it will create all the Speedbuttons which make up the    *
 *                  DBNavigator.                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.InitButtons();
var
  I: TKNGNavBtns;
  Btn: TKNGNavButton;
  X: Integer;
  ResName: string;
  Top : Integer;
begin
  Top := Bevel.BevelSideSize + 1;
  X := Bevel.BevelSideSize + 1;
  MinBtnSize := Point(24, 24);
//  x := (Height - ClientHeight) div 2;
//  X := 0;

  for I := Low(Buttons) to High(Buttons) do begin
    Btn := TKNGNavButton.Create (Self);
    Btn.Flat := Flat;
    Btn.OnClick := ClickHandler;
    Btn.Index := I;
    Btn.Visible := I in FVisibleButtons;
    Btn.Enabled := (I in FEnabledButtons) and (I in FVisibleButtons);
    Btn.SetBounds (X, Top, MinBtnSize.X, MinBtnSize.Y);
    FmtStr(ResName, 'KNGNB_%s', [BtnTypeName[I]]);
    Btn.Glyph.LoadFromResourceName(HInstance, ResName);
    Btn.NumGlyphs := 2;
    Btn.OnMouseDown := BtnMouseDown;
    Btn.Parent := Self;
    Buttons[I] := Btn;
    X := X + MinBtnSize.X
  end;
  Btn.Enabled := False;
  Btn.Enabled := True;
  Btn.OnClick := ClickHandler;
  InitHints;
  Buttons[nbPrior].NavStyle    := Buttons[nbPrior].NavStyle + [nsAllowTimer];
  Buttons[nbNext].NavStyle     := Buttons[nbNext].NavStyle + [nsAllowTimer];
  Buttons[nbPriorSet].NavStyle := Buttons[nbPriorSet].NavStyle + [nsAllowTimer]; {New}
  Buttons[nbNextSet].NavStyle  := Buttons[nbNextSet].NavStyle + [nsAllowTimer] {New}
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.InitHints                                *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 04/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure initialises the hints, it will also store *
 *                  the default hints into the Hints property.               *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.InitHints();
var
  I : Integer;
  J : TKNGNavBtns;
begin
  if not Assigned(FDefaultHints) then
  begin
    FDefaultHints := TStringList.Create;
    for J := Low(Buttons) to High(Buttons) do
      FDefaultHints.Add(LoadResString(BtnHintId[J]));
  end;
  for J := Low(Buttons) to High(Buttons) do
    Buttons[J].Hint := FDefaultHints[Ord(J)];
  J := Low(Buttons);
  for I := 0 to (FHints.Count - 1) do
  begin
    if FHints.Strings[I] <> '' then Buttons[J].Hint := FHints.Strings[I];
    if J = High(Buttons) then Exit;
    Inc(J);
  end;
end;

procedure TKNGDBNavigator.KeyDown(var Key:Word; Shift: TShiftState);
var
  NewFocus : TKNGNavBtns;
  OldFocus : TKNGNavBtns;
begin
  OldFocus := FocusedButton;
  case Key of
    VK_RIGHT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus < High(Buttons) then
            NewFocus := Succ(NewFocus);
        until (NewFocus = High(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate
        end
      end;
    VK_LEFT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus > Low(Buttons) then
            NewFocus := Pred(NewFocus);
        until (NewFocus = Low(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate
        end
      end;
    VK_SPACE:
      begin
        if Buttons[FocusedButton].Enabled then
          Buttons[FocusedButton].Click
      end;
    SB_PAGEUP:
      ShowMessage ('PageUp');
  end;
end;

procedure TKNGDBNavigator.Loaded;
var
  W, H : Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustSize(W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  InitHints;
  ActiveChanged;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.Notification                             *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called each time a copmonent is in-  *
 *                  serted or deleted.  Here we will check if the associated *
 *                  datasource hasn't been deleted and we will clear the     *
 *                  datasource property if necessary                         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Added notification for FocusEditControl delete *
 *                            so that the property will be set to nil when   *
 *                            the component gets deleted.                    *
 * 04/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.Notification(AComponent: TComponent;
                                       Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil)
     and (AComponent = DataSource) then DataSource := nil;
  if (Operation = opRemove) and (AComponent = FFocusEditControl) then
    FFocusEditControl := nil;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetAlign                                 *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 28/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is called when the Align Property is set. *
 *                  It will check which Align it should set and if the Align *
 *                  is alLeft or alRight the Orientation will change to      *
 *                  orVertical.  If the Align is set to alTop or alBottom    *
 *                  then the orientation is set to orHorizontal.             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 28/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetAlign(Value: TAlign);
begin
  FAlign := Value;
  if (FAlign in [alRight, alLeft]) and (FOrientation <> orVertical) then
  begin
    inherited Align := alNone;
    SetOrientation(orVertical);
  end;
  if (FAlign in [alTop, alBottom]) and (FOrientation <> orHorizontal) then
  begin
    inherited Align := alNone;
    SetOrientation(orHorizontal);
  end;
  inherited Align := FAlign;
end;

procedure TKNGDBNavigator.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  OldWidth, OldHeight : Integer;
begin
  OldWidth := AWidth;
  OldHeight := AHeight;
  if not HandleAllocated then AdjustSize(OldWidth, OldHeight);
  { Call the AdjustSize Procedure to adjust the buttons and so }
  inherited SetBounds(ALeft, ATop, OldWidth, OldHeight);
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetDataSource                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 08/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the data-  *
 *                  source property of the component.  It will fill in the   *
 *                  property and trigger the ActiveChanged compoent if ne-   *
 *                  cessary.                                                 *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 08/06/1999 Lesage Stefaan  Corrected a little bug.  When the user sets a  *
 *                            bookmark and then changes the DataSource, the  *
 *                            bookmark wasn't freed so the bookmark was in-  *
 *                            valid, since it still was on the previous data-*
 *                            source.  This should be solved by freeing the  *
 *                            bookmark before changing the Datasource.       *
 *                            Thanks to Juergen Beyer for reporting this.    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetDataSource(Value: TDataSource);
begin
  FreeBookMark;
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then ActiveChanged;
  if Value <> nil then Value.FreeNotification(Self);
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.GetDataSource                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function will return the TDataSource which is stored*
 *                  in the DataSource property of the DBNavigator.           *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBNavigator.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetFlat                                  *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is called when the user sets the Flat     *
 *                  property of our DBNavigator.  It will set the property   *
 *                  and also the Flat property of all the buttons that make  *
 *                  up the DBNavigator.                                      *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetFlat(Value: Boolean);
Var
  I : TKNGNavBtns;
begin
  if FFlat <> Value then
  begin
    FFlat := Value;
    For I := Low(Buttons) to High(Buttons) do
      Buttons[I].Flat := Value;
  End;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetFocusEditControl                      *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the        *
 *                  FocusEditControl property.  It will set the property and *
 *                  make sure that our DBNavigator gets called when the      *
 *                  component gets deleted.                                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetFocusEditControl(Value : TWinControl);
begin
  FFocusEditControl := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetFocusToEditControl                    *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure will make sure that the Control stored in *
 *                  the FocusEditControl property gets the focus (if it can  *
 *                  receive the focus).                                      *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetFocusToEditControl();
begin
  if FFocusEditControl <> nil then
  begin
    with FFocusEditControl do
    begin
      if CanFocus then
      begin
        SetFocus;
      end;
    end;
  end
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetGroupSpace                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the Group- *
 *                  space property.  It will set the property and call the   *
 *                  AdjustSize and SetBounds methods which will redraw the   *
 *                  DBNavigator and it's buttons.                            *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetGroupSpace(Value: Integer);
var
  OldWidth, OldHeight : Integer;
begin
  if FGroupSpace <> Value then
  begin
    FGroupSpace := Value;
    OldWidth := Width;
    OldHeight := Height;
    AdjustSize(OldWidth, OldHeight);
    if (OldWidth <> Width) or (OldHeight <> Height) then
      inherited SetBounds(Left, Top, OldWidth, OldHeight);
    Invalidate;
  end;
end;

procedure TKNGDBNavigator.SetHints(Value: TStrings);
begin
  if Value.Text = FDefaultHints.Text then
    FHints.Clear else
    FHints.Assign(Value);
end;

Function TKNGDBNavigator.GetHints : TStrings;
begin
  if (csDesigning in ComponentState) and not (csWriting in ComponentState) and
     not (csReading in ComponentState) and (FHints.Count = 0) then
    Result := FDefaultHints else
    Result := FHints;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetMoveBySet                             *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the MoveBy *
 *                  property.  It will check if the value is bigger than one *
 *                  raise an error if it isn't the case or set the MoveBy    *
 *                  property when the specified value is bigger than one.    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetMoveBySet(Value: Integer);
begin
  if (Value <= 1) then
    raise Exception.Create(sMoveByTooSmall)
  else
    FMoveBySet := Value;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetOrientation                           *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the Orien- *
 *                  tation property.  It will set the property and call the  *
 *                  SetBounds method which will redraw the DBNavigator and   *
 *                  it's buttons.                                            *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetOrientation(Value: TKNGNavOrientation);
var
  OldWidth, OldHeight : Integer;
begin
  if FOrientation <> Value then
  begin
    FOrientation := Value;
    OldWidth := Width;
    OldHeight := Height;

    if ((OldWidth >= OldHeight) and (Value = orVertical))
    or ((OldWidth <= OldHeight) and (Value = orHorizontal)) then
      inherited SetBounds(Left, Top, OldHeight, OldWidth);
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.SetVisibleButtons                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the Visi-  *
 *                  blebuttons property.  It will set the property and call  *
 *                  the AdjustSize and SetBounds method to redraw the        *
 *                  DBNavigator and it's buttons.                            *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.SetVisibleButtons(Value: TKNGNavBtnSet);
var
  I : TKNGNavBtns;
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FVisibleButtons := Value;
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Visible := I in FVisibleButtons;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Invalidate;
end;

procedure TKNGDBNavigator.UpdateEnabled;
var
  I : TKNGNavBtns;
begin
  for I := Low(Buttons) to High(Buttons) do
    if Buttons[I].Enabled then
      Include(FEnabledButtons, I)
    else
      Exclude(FEnabledButtons, I)
end;

{*****************************************************************************
 * Name           : TKNGDBNavigator.WMSize                                   *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 07/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user changes the size*
 *                  of the DBNavigator component.  It will call the Adjust-  *
 *                  size and SetBounds method to redraw the DBNavigator and  *
 *                  it's buttons.                                            *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 07/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBNavigator.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;

  { check for minimum size }
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0
end;

procedure TKNGDBNavigator.WMSetFocus(var Message: TWMSetFocus);
begin
  Buttons[FocusedButton].Invalidate
end;

procedure TKNGDBNavigator.WMKillFocus(var Message: TWMKillFocus);
begin
  Buttons[FocusedButton].Invalidate
end; 

procedure TKNGDBNavigator.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS
end; 

procedure TKNGDBNavigator.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) then ActiveChanged
end;

{ TKNGNavButton }

destructor TKNGNavButton.Destroy;
begin
  if FRepeatTimer <> nil then FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TKNGNavButton.MouseDown(Button: TMouseButton;
                                  Shift: TShiftState;
                                  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if nsAllowTimer in FNavStyle then
  begin
    if FRepeatTimer = nil then
      FRepeatTimer := TTimer.Create(Self);

    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled := True;
  end;
end;

procedure TKNGNavButton.MouseUp(Button: TMouseButton;
                                Shift: TShiftState;
                                X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then
    FRepeatTimer.Enabled := True;
end;

procedure TKNGNavButton.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (FState = bsDown) and Mousecapture then
  begin
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      raise
    end;
  end;
end;

procedure TKNGNavButton.Paint();
Var
  R : TRect;
begin
  inherited Paint;
  if (GetFocus = Parent.Handle) and
     (FIndex = TKNGDBNavigator(Parent).FocusedButton) then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

{ TKNGNavDataLink }

constructor TKNGNavDataLink.Create(ANav: TKNGDBNavigator);
begin
  inherited Create;
  FNavigator := ANav;
end;

destructor TKNGNavDataLink.Destroy;
begin
  FNavigator := nil;
  inherited Destroy;
end;

procedure TKNGNavDataLink.EditingChanged;
begin
  if FNavigator <> nil then FNavigator.EditingChanged;
end;

procedure TKNGNavDataLink.DataSetChanged;
begin
  if FNavigator <> nil then FNavigator.DataChanged;
end;

procedure TKNGNavDataLink.ActiveChanged();
begin
  if FNavigator <> nil then FNavigator.ActiveChanged;
end;

procedure TKNGDBNavigator.BevelChanged(Sender: TObject);
begin
  inherited bevelchanged(Sender);
  Invalidate;
end;

constructor TKNGDBText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  AutoSize := True;
  ShowAccelChar := False;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
end;

destructor TKNGDBText.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TKNGDBText.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TKNGDBText.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TKNGDBText.SetAutoSize(Value: Boolean);
begin
  if AutoSize <> Value then
  begin
    if Value and FDataLink.DataSourceFixed then DatabaseError(SDataSourceFixed);
    inherited SetAutoSize(Value);
  end;
end;

function TKNGDBText.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TKNGDBText.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TKNGDBText.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TKNGDBText.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TKNGDBText.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TKNGDBText.GetFieldText: string;
begin
  if FDataLink.Field <> nil then
    Result := FDataLink.Field.DisplayText
  else
    if csDesigning in ComponentState then Result := Name else Result := '';
end;

procedure TKNGDBText.DataChange(Sender: TObject);
begin
  Caption := GetFieldText;
end;

function TKNGDBText.GetLabelText: string;
begin
  if csPaintCopy in ControlState then
    Result := GetFieldText else
    Result := Caption;
end;

procedure TKNGDBText.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.Create                                 *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : AOwner.  The owner of the component                      *
 * Return Values  : None                                                     *
 * Description    : This is the constructor of the KNGDBStatusLabel.  In     *
 *                  here we will create other components and set some default*
 *                  properties                                               *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

constructor TKNGDBStatusLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  { Set the Control Style }
  ControlStyle := ControlStyle - [csSetCaption, csReplicatable];

  { Set the DataBaseName property }
  FDatabaseName := '';

  { Create the DataLink }
  FDataLink := TKNGStatusDataLink.Create(Self);
  Height := 13;

  { Set the Caption of the Label }
  Caption := '';
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.Destroy                                *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the Destructor of the component.  In here we     *
 *                  should free all objects we created and clean up.         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

destructor TKNGDBStatusLabel.Destroy;
begin
  { Free the DataLink }
  FDataLink.Free;
  FDataLink := nil;

  { Destroy the Component }
  inherited Destroy;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.GetDatabaseName                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user reads the data- *
 *                  baseName property of the component.  It will return the  *
 *                  DatabaseName                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBStatusLabel.GetDatabaseName: string;
begin
  Result := FDatabaseName;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.SetDatabaseName                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the data-  *
 *                  baseName property, it will set the property and update   *
 *                  the caption of the TKNGDBStatusLabel.                    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.SetDatabaseName(Value: String);
begin
  FDatabasename := Value;

  { Finally Update the Caption }
  UpdateCaption;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.SetShowDatabaseName                    *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the Show-  *
 *                  DatabaseName property.  It will check if the Database-   *
 *                  name property has been set, try to get the default Data- *
 *                  baseName from the associated Dataset and update the      *
 *                  caption.                                                 * 
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.SetShowDatabaseName(Value: Boolean);
begin
  FShowDatabaseName := Value;
  if FShowDatabaseName then
    if FDatabaseName = '' then
      if DataSource.DataSet is TDBDataSet then
        with (DataSource.DataSet as TDBDataSet) do
          FDatabaseName := DatabaseName
  else
    FDatabaseName := '';

  { Finally Update the Caption }
  UpdateCaption;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.GetDataSource                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user reads the Data-  *
 *                  source property.  It will return the datasource of the   *
 *                  associated TDataLink.                                    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBStatusLabel.GetDataSource:TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.SetDataSource                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the Data-  *
 *                  source property.  It will try to get the DatabaseName    *
 *                  of the associated DataSet and store it in the Database-  *
 *                  name property.  It will also update the Caption.         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
  begin
    Value.FreeNotification(Self);
    if Value.DataSet is TDBDataSet then
      SetDatabaseName((Value.DataSet as TDBDataSet).DatabaseName);
  end
  else
  begin
    SetShowDatabaseName(False);
    SetDatabaseName('');
  end;

  if not (csLoading in ComponentState) then UpdateCaption;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.Notification                           *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called each time a copmonent is in-  *
 *                  serted or deleted.  Here we will check if the associated *
 *                  datasource hasn't been deleted and we will clear the     *
 *                  datasource property if necessary                         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource) then DataSource := nil;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.SetName                                *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the Name property gets   *
 *                  set.  It will update the Caption of the KNGDBStatusLabel.*
 *                  This makes sure that when dropping the component on a    *
 *                  form, it doesn't have an empty caption.                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.SetName(const Value: TComponentName);
begin
  inherited SetName(Value);
  if (csDesigning in ComponentState) then UpdateCaption;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.Loaded                                 *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    :                                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.Loaded;
begin
  inherited Loaded;
  UpdateCaption;
end;

{*****************************************************************************
 * Name           : TKNGDBStatusLabel.UpdateCaption                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 15/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the procedure that will update the Caption of    *
 *                  our KNGDBStatusLabel component.  It will check in what   *
 *                  state the associated dataset is and change the Caption   *
 *                  accordingly.                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 15/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBStatusLabel.UpdateCaption();
var
  DBName : string;
begin
  if DataSource <> nil then
  begin
    if (FShowDataBaseName) and
       (FDatabaseName <> '') then
      DBName := (FDatabaseName + ' : ') else DBName := '';
    case DataSource.State of
      dsBrowse :
        if DataSource.DataSet.Filtered then
          Caption := DBName + sBrowseModeFiltered
        else
          Caption := DBName + sBrowseMode;
      dsEdit :
        if DataSource.DataSet.Filtered then
          Caption := DBName + sEditModeFiltered
        else
          Caption := DBName + sEditMode;
      dsInsert :
        if DataSource.DataSet.Filtered then
          Caption := DBName + sInsertModeFiltered
        else
          Caption := DBName + sInsertMode;
    else
      Caption := DBName + sInactiveMode;
    end;
  end
  else
    Caption := sInactiveMode;
//  Invalidate;
end;

{ TKNGDBStatusDataLink }

constructor TKNGStatusDataLink.Create(AKNGDBStatusLabel: TKNGDBStatusLabel);
begin
  inherited Create;
  FKNGDBStatusLabel := AKNGDBStatusLabel;
end;

destructor TKNGStatusDataLink.Destroy;
begin
  FKNGDBStatusLabel := nil;
  inherited destroy;
end;

procedure TKNGStatusDataLink.ActiveChanged;
begin
  DataSetChanged;
end;

procedure TKNGStatusDataLink.DataSetChanged;
begin
  if (FKNGDBStatusLabel <> nil) then FKNGDBStatusLabel.UpdateCaption;
end;

procedure TKNGStatusDataLink.EditingChanged;
begin
  if (FKNGDBStatusLabel <> nil) then FKNGDBStatusLabel.UpdateCaption;
end;

procedure TKNGStatusDataLink.LayoutChanged;
begin
  if (FKNGDBStatusLabel <> nil) then DataSetChanged;
end;

{ TSourceDataLink }

procedure TSourceDataLink.ActiveChanged;
begin
  if FKNGDBLookupLabel <> nil then FKNGDBLookupLabel.DataLinkActiveChanged;
end;

procedure TSourceDataLink.RecordChanged(Field: TField);
begin
  if FKNGDBLookupLabel <> nil then FKNGDBLookupLabel.DataLinkRecordChanged(Field);
end;

procedure TSourceDataLink.FocusControl(Field: TFieldRef);
begin
  if (Field^ <> nil) and (Field^ = FKNGDBLookupLabel.Field) and
    (FKNGDBLookupLabel <> nil) and FKNGDBLookupLabel.FocusControl.CanFocus then
  begin
    Field^ := nil;
    FKNGDBLookupLabel.FocusControl.SetFocus;
  end;
end;

{ TLookupDataLink }

procedure TLookupDataLink.ActiveChanged;
begin
  if FKNGDBLookupLabel <> nil then FKNGDBLookupLabel.ListLinkActiveChanged;
end;

procedure TLookupDataLink.DataSetChanged;
begin
  if FKNGDBLookupLabel <> nil then FKNGDBLookupLabel.ListLinkDataChanged;
end;

{ TKNGDBLookupLabel }

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.Create                                 *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : AOwner.  The owner of the component                      *
 * Return Values  : None                                                     *
 * Description    : This is the constructor of the KNGDBLookupLabel.  In     *
 *                  here we will create other components and set some default*
 *                  properties                                               *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

constructor TKNGDBLookupLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  AutoSize := False;
  ShowAccelChar := False;

  FLookupSource := TDataSource.Create(Self);

  FDataLink := TSourceDataLink.Create;
  FDataLink.FKNGDBLookupLabel := Self;

  FLookupLink := TLookupDataLink.Create;
  FLookupLink.FKNGDBLookupLabel := Self;

  FListFields := TList.Create;

  FEmptyValue:= EmptyStr;
  FValue := FEmptyValue;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.Destroy                                *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This is the Destructor of the component.  In here we     *
 *                  should free all objects we created and clean up.         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

destructor TKNGDBLookupLabel.Destroy;
begin
  FListFields.Free;
  FListFields := nil;

  FLookupLink.FKNGDBLookupLabel := nil;
  FLookupLink.Free;
  FLookupLink := nil;

  FDataLink.FKNGDBLookupLabel := nil;
  FDataLink.Free;
  FDataLink := nil;

  inherited Destroy;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.CheckNotCircular                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure will check if the DataLinks arnt Circular *
 *                  or if they don't refer to eachother.  If that is the     *
 *                  case the sCircularDataLink error will be generated.      *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.CheckNotCircular;
begin
  if (FDataLink.Active) and
     ((DataSource = LookupSource) or
      (FDataLink.DataSet = FLookupLink.DataSet)) then
    { Raise the error if the Datalinks are Circular }
    DatabaseError(SCircularDataLink);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.CheckNotFixed                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure Checks if the DataSource isn't fixed, or  *
 *                  if it can be modified.  If it is Fixed, the sDataSource- *
 *                  Fixed error will be raised.                              *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.CheckNotFixed;
begin
  if FLookupMode then DatabaseError(SPropDefByLookup);
  if FDataLink.DataSourceFixed then DataBaseError(sDataSourceFixed);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.DataLinkActiveChanged                  *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is executed when de DataSource's active   *
 *                  property is modified.  It will check for circular links  *
 *                  and trigger the DataLinkRecordChanged method to update   *
 *                  the KeyValue                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.DataLinkActiveChanged;
begin
  FDataField := nil;
  FMasterField := nil;
  if FDataLink.Active and (FDataFieldName <> '') then
  begin
    CheckNotCircular;
    FDataField := FDataLink.DataSet.FieldByName(FDataFieldName);
    FMasterField := FDataField;
  end;
  SetLookupMode((FDataField <> nil) and FDataField.Lookup);
  DataLinkRecordChanged(nil);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.DataLinkRecordChanged                  *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This method gets called when the DataSet of the Data-    *
 *                  source gets repositioned.  It will call the SetValueKey  *
 *                  procedure so that the value gets changed and the lookup  *
 *                  record gets fetched.                                     *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.DataLinkRecordChanged(Field: TField);
begin
  if (Field = nil) or (Field = FMasterField) then
  begin
    if FMasterField <> nil then
    begin
      SetValueKey(FMasterField.AsString)
    end
    else
      SetValueKey(FEmptyValue);
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetDataSource                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This Function gets called when the value of the data-    *
 *                  source property gets read.                               *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetField                               *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user gets the field   *
 *                  property.                                                *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetField: TField;
begin
  if Assigned(FDataLink) then Result := FDataField else Result := nil;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetKeyValue                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user reads the Key-   *
 *                  value property.                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetKeyValue : Variant;
begin
  if ValueIsEmpty(Value) then VarClear(Result) else Result := Value;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetLabelText                           *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user reads the Caption*
 *                  property.                                                *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetLabelText : string;
begin
  if csPaintCopy in ControlState then
    Result := FDisplayValue else
    Result := Caption;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetLookupField                         *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user reads the Lookup-*
 *                  field property.                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetLookupField : string;
begin
  if FLookupMode then Result := '' else Result := FLookupFieldName;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.GetLookupSource                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This function gets called when the user reads the Lookup-*
 *                  source property.                                         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function TKNGDBLookupLabel.GetLookupSource : TDataSource;
begin
  if FLookupMode then Result := nil else Result := FLookupLink.DataSource;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.ListLinkActiveChanged                  *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the LookupSource's Active*
 *                  property gets changed.                                   *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.ListLinkActiveChanged;
var
  DataSet: TDataSet;
  ResultField: TField;
begin
  FListActive := False;
  FKeyField := nil;
  FDisplayField := nil;
  FListFields.Clear;
  if FLookupLink.Active and (FLookupFieldName <> '') then
  begin
    CheckNotCircular;
    DataSet := FLookupLink.DataSet;
    FKeyField := DataSet.FieldByName(FLookupFieldName);
    DataSet.GetFieldList(FListFields, FLookupDisplay);
    if FLookupMode then
    begin
      ResultField := DataSet.FieldByName(FDataField.LookupResultField);
      if FListFields.IndexOf(ResultField) < 0 then
        FListFields.Insert(0, ResultField);
      FDisplayField := ResultField;
    end
    else
    begin
      if FListFields.Count = 0 then FListFields.Add(FKeyField);
      if (FDisplayIndex >= 0) and (FDisplayIndex < FListFields.Count) then
        FDisplayField := FListFields[FDisplayIndex] else
        FDisplayField := FListFields[0];
    end;
    FListActive := True;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.Notification                           *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when a componets is added or  *
 *                  removed from the form.  In here we will check if our     *
 *                  datasources havn't been deleted, if that is the case, we *
 *                  will clear the appropriate Datasource property.          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if (FDatalink <> nil) and (AComponent = DataSource) then DataSource := nil;
    if (FLookuplink <> nil) and (AComponent = LookupSource) then LookupSource := nil;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetDataFieldName                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the user sets the data-  *
 *                  field property.                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetDataFieldName(const Value: string);
begin
  if FDataFieldName <> Value then
  begin
    FDataFieldName := Value;
    DataLinkActiveChanged;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetDataSource                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure get called when the user Sets the Data-   *
 *                  source property.  It will also make sure we get noti-    *
 *                  fied if the datasource gets removed.                     *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetDisplayEmpty                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the DisplayEmpty proper- *
 *                  ty gets set.                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetDisplayEmpty(const Value: string);
begin
  if FDisplayEmpty <> Value then
  begin
    UpdateDisplayEmpty(Value);
    FDisplayEmpty := Value;
    if not (csReading in ComponentState) then Invalidate;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetEmptyValue                          *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the EmptyValue property  *
 *                  gets set.                                             *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetEmptyValue(const Value: string);
begin
  if FEmptyValue <> Value then
  begin
    if ValueIsEmpty(FValue) then FValue := Value;
    FEmptyValue := Value;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetLookupDisplay                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    :                                                          *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetLookupDisplay(const Value: string);
begin
  if FLookupDisplay <> Value then
  begin
    FLookupDisplay := Value;
    ListLinkActiveChanged;
    if FListActive then DataLinkRecordChanged(nil);
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetLookupField                         *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the LookupField property *
 *                  get set.  It will call the ListLinkActiveChanged and     *
 *                  DataLinkRecordChanged methods so that the lookup record  *
 *                  gets fetched.                                            *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetLookupField(const Value: string);
begin
  CheckNotFixed;
  if FLookupFieldName <> Value then
  begin
    FLookupFieldName := Value;
    ListLinkActiveChanged;
    if FListActive then DataLinkRecordChanged(nil);
  end;
end;

procedure TKNGDBLookupLabel.SetLookupMode(Value: Boolean);
begin
  if FLookupMode <> Value then
    if Value then
    begin
      FMasterField := FDataField.DataSet.FieldByName(FDataField.KeyFields);
      FLookupSource.DataSet := FDataField.LookupDataSet;
      FLookupFieldName := FDataField.LookupKeyFields;
      FLookupMode := True;
      FLookupLink.DataSource := FLookupSource;
    end
    else
    begin
      FLookupLink.DataSource := nil;
      FLookupMode := False;
      FLookupFieldName := '';
      FLookupSource.DataSet := nil;
      FMasterField := FDataField;
    end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetLookupSource                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the Lookup DataSource is *
 *                  set (LookupSource property).  It will check for a circ-  *
 *                  cular datalink and make sure we get notified when the    *
 *                  datasource gets removed.                                 *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetLookupSource(Value: TDataSource);
begin
  CheckNotFixed;
  FLookupLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if FListActive then DataLinkRecordChanged (nil);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.SetValueKey                            *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called each time the Master record   *
 *                  gets changed (when de FDataSource.DataSet moves, ...)    *
 *                  It will call the KeyValueChanged to fetch the Lookup     *
 *                  record and update the control.                           *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.SetValueKey(const Value: string);
begin
  if FValue <> Value then
  begin
    FValue := Value;
    KeyValueChanged;
  end;
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.UpdateDisplayEmpty                     *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure is called each time the DisplayEmpty pro- *
 *                  perty is modified.  It will check if the current Caption *
 *                  is the DisplayEmpty property.  If that is the case, it   *
 *                  will update the caption with the new DisplayEmpty text.  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.UpdateDisplayEmpty(const Value: string);
begin
  if Caption = FDisplayEmpty then inherited Caption := Value;
end;

Function TKNGDBLookupLabel.ValueIsEmpty(const s: string): Boolean;
begin
  Result := (s = FEmptyValue);
end;

{*****************************************************************************
 * Name           : TKNGDBLookupLabel.KeyValueChanged                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 23/06/1999                                               *
 * Arguments      : None                                                     *
 * Return Values  : None                                                     *
 * Description    : This procedure gets called when the KeyValue gets changed*
 *                  it will locate the LookupRecord and update the caption   *
 *                  of the component                                         *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 23/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

procedure TKNGDBLookupLabel.KeyValueChanged();
var
  Bookmark : TBookmark;
begin
  if not FLookupLink.Active then FLookupLink.dataset.Active := True;
  FLookupLink.DataSet.CheckBrowseMode;
  FLookupLink.DataSet.CursorPosChanged;
  BookMark := FLookupLink.DataSet.GetBookmark;
  FLookupLink.DataSet.DisableControls;
  if FLookupLink.DataSet.Locate(FLookupFieldName, FValue, []) then
    FDisplayValue := FLookupLink.DataSet.FieldByName(FLookupDisplay).AsString
  else
    FDisplayValue := 'Lookup Recrod Value not Found';
  FLookupLink.DataSet.EnableControls;
  FLookupLink.DataSet.GotoBookmark(Bookmark);
  FLookupLink.DataSet.FreeBookMark(Bookmark);

  Caption := FDisplayValue;
end;

procedure TKNGDBLookupLabel.ListLinkDataChanged;
begin
end;

end.
